package speiger.src.coreengine.rendering.utils;

import org.lwjgl.opengl.GL11;

import speiger.src.coreengine.math.collision2d.Plane;
import speiger.src.coreengine.utils.io.GameLog;

public class ScissorsManager
{
	Plane[] instances;
	int inUse = -1;
	
	public ScissorsManager(int limit)
	{
		instances = new Plane[limit];
		for(int i = 0;i<limit;i++)
		{
			instances[i] = new Plane();
		}
	}
	
	public void enableScissors(int x, int y, int width, int height)
	{
		if(inUse + 1 >= instances.length)
		{
			GameLog.warn("Scissors test would go out of Bounds!");
			return;
		}
		inUse++;
		if(inUse == 0)
		{
			instances[inUse].set(x, y, x + width, y + height);
			GL11.glEnable(GL11.GL_SCISSOR_TEST);
			GL11.glScissor(x, y, width, height);
			return;
		}
		Plane oldBox = instances[inUse - 1];
		int maxX = x + width;
		int maxY = y + height;
        if (x < oldBox.getMinX()) x = oldBox.getMinX();
        if (maxX > oldBox.getMaxX()) maxX = oldBox.getMaxX();
        if (y < oldBox.getMinY()) y = oldBox.getMinY();
        if (maxY > oldBox.getMaxY()) maxY = oldBox.getMaxY();
		Plane box = instances[inUse];
		box.set(x, y, maxX, maxY);
		GL11.glScissor(box.getMinX(), box.getMinY(), box.getWidth(), box.getHeight());
	}
	
	public boolean isInScissors(int x, int y, int width, int height)
	{
		if(inUse == -1)
		{
			return true;
		}
		return instances[inUse].isIntersecting(x, y, x + width, y + height);
	}
	
	public void disableScissors()
	{
		if(inUse < 0)
		{
			GameLog.warn("Scissors test would go into negative");
			return;
		}
		inUse--;
		if(inUse >= 0)
		{
			Plane box = instances[inUse];
			GL11.glScissor(box.getMinX(), box.getMinY(), box.getWidth(), box.getHeight());
		}
		else
		{
			GL11.glDisable(GL11.GL_SCISSOR_TEST);
		}
	}
}
